using System;
using System.Data;
using System.Configuration;
using System.Data.Common;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib.Cart
{
    public class AdminMenuPageDetails
    {
        private string _menuPageName;
        private string _menuParentPageName;
        private bool _isEnabled;
        private string _reliantOn;
        private string _breadcrumbName;

        public string MenuPageName { get { return _menuPageName; } set { _menuPageName = value; } }
        public string MenuParentPageName { get { return _menuParentPageName; } set { _menuParentPageName = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public string ReliantOn { get { return _reliantOn; } set { _reliantOn = value; } }
        public string BreadcrumbName { get { return _breadcrumbName; } set { _breadcrumbName = value; } }
    }

    public static class AdminMenuPageAccess
    {
        private const string AllMenuPageFields = "AdminMenuPage.MenuPageName, ParentMenuPageName, IsEnabled, ReliantOn, BreadcrumbName, MenuDescription ";

        public static DataTable GetAll( string cultureID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllMenuPageFields + " " +
                "FROM AdminMenuPage " +
                "LEFT JOIN (SELECT * FROM AdminMenuPageLocale WHERE CultureID = @CultureID) AS L " +
                "ON AdminMenuPage.MenuPageName = L.MenuPageName " +
                "ORDER BY AdminMenuPage.MenuPageName; ",
                DataAccess.CreateParameterString( cultureID )
            );

            return table;
        }

        public static DataTable GetTableSchema()
        {
            string sql =
                "SELECT " + AllMenuPageFields + " " +
                "FROM AdminMenuPage WHERE  1 = 0 ";

            DataTable table = DataAccess.ExecuteSelect( sql );
            DataColumn[] keys = new DataColumn[] { table.Columns["MenuPageName"] };
            table.PrimaryKey = keys;
            return table;
        }

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        public static void Create(
            string cultureID,
            string menuPageName,
            string menuParentPageName,
            string breadcrumbName,
            string menuDescription,
            bool isEnabled,
            string reliantOn
        )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO AdminMenuPage ( MenuPageName, MenuParentPageName, IsEnabled, ReliantOn ) " +
                "VALUES (@MenuPageName, @MenuParentPageName, @IsEnabled, @ReliantOn ); ",
                DataAccess.CreateParameterString( menuPageName ),
                DataAccess.CreateParameterString( menuParentPageName ),
                DataAccess.CreateParameterBool( isEnabled ),
                DataAccess.CreateParameterString( reliantOn )
                );

            DataAccess.ExecuteNonQuery(
                "INSERT INTO AdminMenuPageLocale ( CultureID, MenuPageName, BreadcrumbName, MenuDescription ) VALUES (" +
                " @CultureID, @MenuPageName, @BreadcrumbName, @MenuDescription );",
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( menuPageName ),
                DataAccess.CreateParameterString( breadcrumbName ),
                DataAccess.CreateParameterString( menuDescription ) );
        }
    }
}
