using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Caching;
using System.Collections;

namespace Vevo.Base.Domain
{
    public class CacheDependencyUtility : ICacheDependencyUtility
    {
        #region Private
        private SqlCacheDependency _sqlCacheDependency = null;
        private AggregateCacheDependency _aggregateCacheDependency = null;
        private List<string> _tableNameList = new List<string>();
        private string _tableName;
        private bool _isSqlCacheDependency = true;
        private const string connName = "StoreDB";

        private void SetSqlCacheDependency()
        {
            _isSqlCacheDependency = true;

            _sqlCacheDependency = new SqlCacheDependency( connName, _tableName );
        }

        private void SetAggregateCacheDependency()
        {
            _isSqlCacheDependency = false;

            _aggregateCacheDependency = new AggregateCacheDependency();

            foreach (string tableName in _tableNameList)
            {
                SqlCacheDependency sqlCachDependency = new SqlCacheDependency( connName, tableName );
                _aggregateCacheDependency.Add( sqlCachDependency );
            }
        }

        private bool DependencyUpdateHelper()
        {
            bool isHasChanged = false;

            if (_isSqlCacheDependency)
            {
                if (_sqlCacheDependency == null)
                {
                    SetSqlCacheDependency();
                    isHasChanged = true;
                }
                else
                {
                    isHasChanged = _sqlCacheDependency.HasChanged;

                    if (isHasChanged)
                    {
                        SetSqlCacheDependency();
                    }
                }
            }
            else
            {
                if (_aggregateCacheDependency == null)
                {
                    SetAggregateCacheDependency();
                    isHasChanged = true;
                }
                else
                {
                    isHasChanged = _aggregateCacheDependency.HasChanged;

                    if (isHasChanged)
                    {
                        SetAggregateCacheDependency();
                    }
                }
            }

            return isHasChanged;
        }
        #endregion

        #region Public Constructors

        public CacheDependencyUtility()
        {

        }

        public CacheDependencyUtility( string tableName )
        {
            try
            {
                _tableName = tableName;
                SetSqlCacheDependency();
            }
            catch { }
        }

        public CacheDependencyUtility( string[] tableNameList )
        {
            try
            {
                _tableNameList.Clear();

                foreach (string tableName in tableNameList)
                {
                    _tableNameList.Add( tableName );
                }
                SetAggregateCacheDependency();
            }
            catch { }
        }
        #endregion

        #region ICacheDependencyUtility Members

        bool ICacheDependencyUtility.getHasChanged
        {
            get
            {
                if (KeyUtilities.IsMultistoreLicense())
                    return DependencyUpdateHelper();
                else
                    return false;
            }
        }

        #endregion

        #region Public Method

        public void CreateSqlCacheTableForNotifications( string connectionString )
        {
            // enable sql cache dependency support in the database if needed
            // this will fail if the account does not have adminstrator permissions on the Pubs database 
            SqlCacheDependencyAdmin.EnableNotifications( connectionString );
            string[] tableEnabledNotificationList =
                SqlCacheDependencyAdmin.GetTablesEnabledForNotifications( connectionString );

            string[] tableNotificationList = new string[] 
            {
                "Admin",
                "AdminMenuAdvanced",
                "AdminMenuAdvancedLocale",
                "AdminMenuAdvancedPermission",
                "AdminMenuAdvancedPermissionLocale",
                "AdminMenuClassic",
                "AdminMenuClassicLocale",
                "AdminMenuPage",
                "AdminMenuPageLocale",
                "AdminMenuPermission",
                "Affiliate",
                "AffiliateOrders",
                "AffiliatePayment",
                "aspnet_Membership",
                "aspnet_Profile",
                "aspnet_Users",
                "aspnet_UsersInRoles",
                "Banner",
                "BannerLocale",
                "Blog",
                "BlogCategory",
                "BlogCategoryLocale",
                "BlogCategoryItem",
                "BlogStore",
                "BlogComment",
                "Cart",
                "CartItem",
                "Category",
                "CategoryLocale",
                "Configuration",
                "ConfigurationDescription",
                "ConfigurationValue",
                "Content",
                "ContentLocale",
                "ContentMenu",
                "ContentMenuItem",
                "ContentMenuItemLocale",
                "Country",
                "Coupon",
                "Culture",
                "Currency",
                "Customer",
                "CustomerReview",
                "CustomerRewardPoint",
                "CustomerSubscription",
                "Department",
                "DepartmentLocale",
                "DiscountGroup",
                "DiscountRule",
                "EBayTemplate",
                "EmailTemplateDetail",
                "EmailTemplateDetailLocale",
                "GiftCertificate",
                "GiftRegistry",
                "GiftRegistryItem",
                "Help",
                "LanguageText",
                "ManufacturerLocale", 
                "Manufacturer",
                "News",
                "Newsletter",
                "NewsLocale",
                "OptionCombination",
                "OptionGroup",
                "OptionGroupLocale",
                "OptionItem",
                "OptionItemLocale",
                "OrderItem",
                "OrderItemRecurring",
                "OrderNote",
                "Orders",
                "OrderTracking",
                "Page",
                "PaymentLog",
                "PaymentOption",
                "PaymentOptionLocale",
                "PayPalIPN",
                "Product",
                "ProductCategory",
                "ProductCustomPrice",
                "ProductDepartment",
                "ProductGiftCertificate",
                "ProductImage",
                "ProductKit",
                "ProductKitGroup",
                "ProductKitGroupLocale",
                "ProductKitItem",
                "ProductLocale",
                "ProductOptionGroup",
                "ProductPrice",
                "ProductRecurring",
                "ProductShippingCost",
                "ProductSpecification",
                "ProductStock",
                "ProductSubscription",
                "PromotionGroup",
                "PromotionGroupLocale",
                "PromotionGroupSubGroup",
                "PromotionProduct",
                "PromotionSubGroup",
                "ProductMetaInformation",
                "QuickBooksLog",
                "QuickBooksLogItem",
                "QuickBooksLogSummary",
                "RecurringProfile",
                "Rma",
                "RmaAction",
                "SearchLog",
                "Shipping",
                "ShippingAddress",
                "ShippingLocale",
                "ShippingOptionType",
                "ShippingWeightRate",
                "ShippingOrderTotalRate",
                "ShippingZone",
                "ShippingZoneGroup",
                "ShippingZoneItem",
                "SpecificationGroup",
                "SpecificationGroupLocale",
                "SpecificationItem",
                "SpecificationItemLocale",
                "State",
                "Store",
                "SubscriptionLevel",
                "TaxClass",
                "TaxClassRule",
                "WishList"
            };

            foreach (string tableNotification in tableNotificationList)
            {
                bool _flagExists = false;

                foreach (string tableEnabledNotification in tableEnabledNotificationList)
                {
                    if (tableEnabledNotification.Equals( tableNotification ))
                    {
                        _flagExists = true;
                    }
                }

                if (!_flagExists)
                {
                    SqlCacheDependencyAdmin.EnableTableForNotifications( connectionString, tableNotification );
                }
            }
        }

        #endregion
    }
}
