using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Orders;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CartItemRepository : ICartItemRepository
    {
        #region Private

        private const string RepositoryName = "CartItemRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetByCartIDSignature = "GetByCartID";
        private const string TableName = "CartItem";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private CartItemRepositoryNoCache _repository = new CartItemRepositoryNoCache();

        #endregion

        #region Constructors

        public CartItemRepository()
        {
        }

        public CartItemRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region ICartItemRepository Members

        public ICartItem Create( ICartItem cartItem )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ICartItem newCartItem = _repository.Create( cartItem );
            _dataCache.ClearRepositoryCache();
            return newCartItem;
        }

        public ICartItem Update( ICartItem cartItem )
        {
            ICartItem result = _repository.Update( cartItem );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public void Delete( string cartItemID )
        {
            _repository.Delete( cartItemID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByCartID( string cartID )
        {
            _repository.DeleteByCartID( cartID );
            _dataCache.ClearRepositoryCache();
        }

        public ICartItem GetOne( string cartItemID)
        {
            ICartItem result = (ICartItem) _dataCache.GetValue( GetOneSignature, _cacheDependency, cartItemID);

            if (result == null)
            {
                result = _repository.GetOne( cartItemID);
                _dataCache.SetValue( GetOneSignature, result, cartItemID);
            }

            return result;
        }

        public IList<ICartItem> GetAllByCartID( string cartID)
        {
            IList<ICartItem> list = (IList<ICartItem>) _dataCache.GetValue(
                    GetByCartIDSignature, _cacheDependency, cartID);

            if (list == null)
            {
                list = _repository.GetAllByCartID( cartID);
                _dataCache.SetValue(
                    GetByCartIDSignature,
                    list,
                    cartID
                    );
            }

            return list;
        }

        #endregion
    }
}
