﻿using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Returns;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class RmaRepository : IRmaRepository
    {
        #region Private

        private const string RepositoryName = "RmaRepository";

        private const string GetOneSignature = "GetOne";
        private const string SearchRmaSignature = "SearchRma";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetRmaHistoryTableSchemaSignature = "RmaHistoryTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( "Rma" );
        private RmaRepositoryNoCache _repository = new RmaRepositoryNoCache();

        #endregion

        #region Constructors

        public RmaRepository()
        {

        }

        public RmaRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Rma Save( Rma rma )
        {
            Rma newRma = _repository.Save( rma );
            _dataCache.ClearRepositoryCache();
            return newRma;
        }

        public Rma GetOne( string rmaID )
        {
            Rma result = ( Rma ) _dataCache.GetValue( GetOneSignature, _cacheDependency, rmaID );

            if ( result == null )
            {
                result = _repository.GetOne( rmaID );
                _dataCache.SetValue( GetOneSignature, result, rmaID );
            }

            return result;
        }

        public void Delete( string rmaID )
        {
            _repository.Delete( rmaID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = ( Pair ) _dataCache.GetValue(
                SearchRmaSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if ( result == null )
            {
                result = new Pair();
                result.First
                    = _repository.SearchRma( sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchRmaSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return ( IList<Rma> ) result.First;
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            string storeID,
            string customerID,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = ( Pair ) _dataCache.GetValue(
                SearchRmaSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                storeID,
                customerID
                );

            if ( result == null )
            {
                result = new Pair();
                result.First
                    = _repository.SearchRma( sortBy, searchFilter, storeID, customerID, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchRmaSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    storeID,
                    customerID );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return ( IList<Rma> ) result.First;
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            Rma.RmaStatus status,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = ( Pair ) _dataCache.GetValue(
                SearchRmaSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex,
                storeID,
                status
                );

            if ( result == null )
            {
                result = new Pair();
                result.First
                    = _repository.SearchRma( sortBy, searchFilter, startIndex, endIndex, storeID, status, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchRmaSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex,
                    storeID,
                    status );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return ( IList<Rma> ) result.First;
        }

        public IList<Rma> SearchRma(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            string storeID,
            string customerID,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = ( Pair ) _dataCache.GetValue(
                SearchRmaSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex,
                storeID,
                customerID
                );

            if ( result == null )
            {
                result = new Pair();
                result.First
                    = _repository.SearchRma( sortBy, searchFilter, startIndex, endIndex, storeID, customerID, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchRmaSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex,
                    storeID,
                    customerID );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return ( IList<Rma> ) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = ( IList<TableSchemaItem> ) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if ( list == null )
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        public IList<TableSchemaItem> GetRmaHistoryTableSchema()
        {
            IList<TableSchemaItem> list = ( IList<TableSchemaItem> ) _dataCache.GetValue( GetRmaHistoryTableSchemaSignature, _cacheDependency );

            if ( list == null )
            {
                list = _repository.GetRmaHistoryTableSchema();
                _dataCache.SetValue( GetRmaHistoryTableSchemaSignature, list );
            }

            return list;
        }

        #endregion
    }
}
