using System;
using System.Data;
using System.Configuration;

using Vevo.Domain.Configurations;

namespace Vevo.Domain.Tax
{
    /// <summary>
    /// Summary description for IncludedTaxCalculator
    /// </summary>
    public class IncludedTaxCalculator
    {
        #region Private

        private ConfigurationCollection _configurations;


        private double GetTaxRatio()
        {
            return _configurations.GetDoubleValue( "TaxPercentageIncludedInPrice" ) / 100.0;
        }

        #endregion


        #region Public Methods

        public IncludedTaxCalculator(
            ConfigurationCollection configurations )
        {
            _configurations = configurations;
        }

        public decimal GetAmount( decimal priceWithTax, bool isTaxable )
        {
            if (!_configurations.GetBoolValue( "TaxIncludedInPrice" ))
                return 0;

            if (!isTaxable)
                return 0;

            double taxRate = GetTaxRatio();
            double taxFraction = taxRate / (1 + taxRate);

            return Currency.RoundAmount( priceWithTax * (decimal) taxFraction );
        }

        #endregion

    }
}
