using System;
using Vevo.Domain;
using Vevo.QuickBooks.Qbxml70;
using Vevo.Domain.Users;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for CustomerAddResponseState
    /// </summary>
    public class CustomerAddResponseState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new ProductVerifyRequestState( QBWCSession );
        }

        private void ProcessRecieve( string response )
        {
            QBXMLMsgsRs qbXMLMsgsRs = CreateResponse( response );

            int successCount = 0;
            int failureCount = 0;

            for (int i = 0; i < qbXMLMsgsRs.Items.Length; i++)
            {
                CustomerAddRsType result = (CustomerAddRsType) qbXMLMsgsRs.Items[i];

                string customerID = string.Empty;
                if (result.statusCode.ToString() == "0")
                {
                    string[] notes = result.CustomerRet.Notes.Split( ' ', ',' );
                    customerID = notes[1].Trim();
                    if (customerID == "0")
                    {
                        DataAccessContext.ConfigurationRepository.UpdateValue(
                            DataAccessContext.Configurations["QBAnonymousListID"],
                            result.CustomerRet.ListID );
                    }
                    else
                    {
                        Customer customer = DataAccessContext.CustomerRepository.GetOne( customerID );
                        customer.QBListID = result.CustomerRet.ListID;
                        customer.IsQBExported = true;
                        DataAccessContext.CustomerRepository.Save( customer );
                    }
                    successCount++;
                }
                else
                {
                    CreateQBLogItem( result.statusCode, result.statusSeverity, result.statusMessage, QBExportType.CustomerAdd );
                    failureCount++;
                }
            }

            CreateQBLogSummary( QBExportType.CustomerAdd, successCount, failureCount );
        }

        public CustomerAddResponseState( QBWCSession session )
            : base( session )
        {
        }

        public override int OnReceiveResponseXml( string responseXml )
        {
            ProcessRecieve( responseXml );
            QBWCSession.ClearList();
            MoveToNextState();
            return 32;
        }

    }
}
