using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.WebAppLib;

namespace Vevo.WebUI.DataAccessCache
{
    public interface IPageAccessCache
    {
        string GetIDByPath( string path );
    }


    /// <summary>
    /// Summary description for PageAccessCache
    /// </summary>
    public static class PageAccessCache
    {
        private static CachedObject _cachedObject = new CachedObject();
        private static NonCachedObject _nonCachedObject = new NonCachedObject();


        public static IPageAccessCache Instance
        {
            get
            {
                if (DataAccessContext.Configurations.GetBoolValueNoThrow( "LanguageCachingEnabled" ))
                    return _cachedObject;
                else
                    return _nonCachedObject;
            }
        }

        public static void ClearCache()
        {
            _cachedObject.ClearCache();
        }


        //----------------------------------------------------------------------------------
        // CachedObject
        //----------------------------------------------------------------------------------

        private class CachedObject : BaseDataAccessCache, IPageAccessCache
        {
            private const string PageTableCacheName = "PageAccessCache_PathToID";


            private DataTable CachedPageTable
            {
                get
                {
                    DataTable table = (DataTable) HttpRuntime.Cache[PageTableCacheName];

                    if (table == null)
                    {
                        table = PageAccess.GetAll( "Path" );
                        HttpRuntime.Cache.Insert(
                            PageTableCacheName, table, null,
                            Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration,
                            CacheItemPriority.AboveNormal, null );
                    }

                    return table;
                }
            }


            public CachedObject()
                : base( "PageAccess" )
            {
            }

            public override void ClearCache()
            {
                base.ClearCache();

                HttpRuntime.Cache.Remove( PageTableCacheName );
            }


            public string GetIDByPath( string path )
            {
                foreach (DataRow row in CachedPageTable.Rows)
                {
                    if (string.Compare( row["Path"].ToString(), path, true ) == 0)
                        return row["PageID"].ToString();
                }

                return "0";
            }

        }

        //----------------------------------------------------------------------------------
        // NonCachedObject
        //----------------------------------------------------------------------------------

        private class NonCachedObject : IPageAccessCache
        {
            public string GetIDByPath( string path )
            {
                return PageAccess.GetIDByPath( path );
            }

        }

    }
}
